#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "huffman.h"

static void
dump_tree(const char *name, unsigned n, const struct huffman_node *tree)
{
  unsigned k;
  
  printf("const struct huffman_node %s[] = {", name);
  for (k = 0; k < n; k++)
    {
      if (!(k & 3))
	printf("\n  ");
      printf("{%d,%d,%d},", tree[k].zeros, tree[k].leaf, tree[k].value);
    }
  printf("\n};\n");
}

/* Includes the two unused codes 286 and 287. */
#define LITERAL_SIZE 288

/* In RFC1951, the coding is defined to include codes for the invalid
   values 30 and 31. We use a decoding tree that does not include
   these values. Unlike the length/literal coding, deleting these
   invalid values does not change the coding of any valid values. */
#define DISTANCE_SIZE 30

int
main (int argc, char **argv)
{
  struct huffman_node tree[HUFFMAN_SIZE(LITERAL_SIZE)];
  unsigned char clen[LITERAL_SIZE];

  /* Literal and length code. */
  memset(clen, 8, 144);
  memset(clen + 144, 9, 256 - 144);
  memset(clen + 256, 7, 280 - 256);
  memset(clen + 280, 8, LITERAL_SIZE - 280);

  if (!huffman_tree (tree, LITERAL_SIZE, clen))
    {
      fprintf(stderr, "Failed to generate length/literal code.\n");
      return EXIT_FAILURE;
    }
     
  dump_tree ("fixed_length_tree", HUFFMAN_SIZE(LITERAL_SIZE), tree);

  memset(clen, 5, DISTANCE_SIZE);

  if (!huffman_tree (tree, DISTANCE_SIZE, clen))
    {
      fprintf(stderr, "Failed to generate distance code.\n");
      return EXIT_FAILURE;
    }

  dump_tree ("fixed_distance_tree", HUFFMAN_SIZE(DISTANCE_SIZE), tree);
  
  return 0;
};
