/* Simple huffman decoding. */

/* Copyright (C) 2009 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INFLATE_HUFFMAN_H_INCLUDED
#define INFLATE_HUFFMAN_H_INCLUDED

/* Huffman decoding tree is represented as a binary tree, represented
 * as a linear array of nodes. */

/* FIXME: Use bitfields, or some other trickery to fit in 16 bits? */

struct huffman_node {
  /* Extra zero bits that are required when entering this node. (If input
     doesn't match, it's a decoding error). */
  unsigned char zeros;

  /* Set if this is a leaf node. */
  unsigned char leaf;

  /* For a leaf node, we have a complete codeword corresponding to VALUE.
     For an interior node, the left subtree is always in the next
     entry in the array, while the offset to the right subtree is
     given by VALUE. */
  unsigned short value;
};

struct huffman_ctx {
  const struct huffman_node *root;
  const struct huffman_node *p;
  unsigned zeros;
};

struct bit_buffer
{
  /* Number of bits. */
  unsigned n;
  /* Buffered bits, starting at least significant end. */
  unsigned bits;
};

/* A binary tree with N leafs needs N-1 non-leaf nodes (the interior nodes
 * have a total of 2(N-1) child pointers, out of which N-2 points at
 * interior nodes, leaving 2(N-1) - (N-2) = N pointers to leafs). The
 * total number of nodes is then 2N - 1. */

#define HUFFMAN_SIZE(n) (2*(n) - 1)

int
huffman_tree (struct huffman_node *tree,
	      unsigned n, const unsigned char *lengths);


void
huffman_init (struct huffman_ctx *ctx,
	      const struct huffman_node *root);

/* Returns -1 on error, 0 when a partial code word is read, and 1 if a
   compete code word is read, in which case the value is stored in
   *VALUE. */
int
huffman_decode_bits (struct huffman_ctx *ctx,
		     struct bit_buffer *input,
		     unsigned *value);

#endif /* INFLATE_HUFFMAN_H_INCLUDED */
